# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0003_auto_20250704_0810'),
        ('edu_rdm_integration_models', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='RegionalDataMartModelEnum',
            new_name='RDMModelEnum',
        ),
    ]
