# Generated by Django 3.2.24 on 2025-05-07 19:50

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration', '0016_transferredentity_queue_level'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UploadDataCommand',
        ),
    ]
