# Generated by Django 2.2.28 on 2023-12-01 20:58
from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('uploader_client', '0001_initial'),
        ('edu_rdm_integration', '0003_create_index_file_upload_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='UploaderClientLog',
            fields=[],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('edu_rdm_integration.exportingdatasubstageuploaderclientlog',),
        ),
    ]
