# Generated by Django 3.2.24 on 2025-07-04 08:25

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0003_auto_20250704_0810'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rdmcollectingdatasubstage',
            options={'verbose_name': 'Подэтап сбора данных', 'verbose_name_plural': 'Подэтапы сбора данных'},
        ),
        migrations.AlterField(
            model_name='rdmcollectingdatacommandprogress',
            name='stage',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to='edu_rdm_integration_collect_data_stage.rdmcollectingdatastage',
                verbose_name='Этап сбора данных',
            ),
        ),
        migrations.AlterField(
            model_name='rdmcollectingdatasubstage',
            name='stage',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to='edu_rdm_integration_collect_data_stage.rdmcollectingdatastage',
                verbose_name='Этап сбора данных',
            ),
        ),
    ]
