# Generated by Django 3.2.24 on 2025-10-08 14:17

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):

    dependencies = [
        ('uploader_client', '0001_initial'),
        ('edu_rdm_integration_upload_data_stage', '0002_auto_20250704_0810'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rdmexportingdatasubstageuploaderclientlog',
            name='entry',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='uploader_client_log', to='uploader_client.entry', verbose_name='Лог запроса и ответа'),
        ),
        migrations.AlterField(
            model_name='rdmuploadstatusrequestlog',
            name='entry',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='upload_status_request_log', to='uploader_client.entry', verbose_name='Cвязь запроса статуса с запросом в витрину'),
        ),
    ]
