from edu_function_tools.errors import (
    EduError,
)


class BaseCollectingExportedDataError(EduError):
    """Базовая ошибка функций сбора данных для интеграции с "Региональная витрина данных"."""
