# Generated by Django 3.1.14 on 2024-08-19 14:24

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration', '0011_exportingdatasubstageentity'),
    ]

    operations = [
        migrations.AddField(
            model_name='exportingdatasubstageattachment',
            name='attachment_size',
            field=models.PositiveIntegerField(null=True, verbose_name='Размер файла (байт)'),
        ),
    ]
