# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_export_data_stage', '0002_auto_20250704_0810'),
        ('uploader_client', '0001_initial'),
        ('edu_rdm_integration_upload_data_stage', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ExportingDataSubStageUploaderClientLog',
            new_name='RDMExportingDataSubStageUploaderClientLog',
        ),
        migrations.RenameModel(
            old_name='DataMartRequestStatus',
            new_name='RDMRequestStatus',
        ),
        migrations.RenameModel(
            old_name='UploadStatusRequestLog',
            new_name='RDMUploadStatusRequestLog',
        ),
    ]
