# Generated by Django 3.2.24 on 2025-07-04 07:25

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('rdm_collect_and_export_data', '0003_auto_20250704_0725'),
        ('edu_rdm_integration', '0017_delete_uploaddatacommand'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.RemoveField(
                    model_name='collectingexporteddatastage',
                    name='manager',
                ),
                migrations.RemoveField(
                    model_name='collectingexporteddatastage',
                    name='status',
                ),
                migrations.RemoveField(
                    model_name='collectingexporteddatasubstage',
                    name='function',
                ),
                migrations.RemoveField(
                    model_name='collectingexporteddatasubstage',
                    name='previous',
                ),
                migrations.RemoveField(
                    model_name='collectingexporteddatasubstage',
                    name='stage',
                ),
                migrations.RemoveField(
                    model_name='collectingexporteddatasubstage',
                    name='status',
                ),
                migrations.RemoveField(
                    model_name='exportingdatastage',
                    name='manager',
                ),
                migrations.RemoveField(
                    model_name='exportingdatastage',
                    name='status',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstage',
                    name='function',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstage',
                    name='stage',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstage',
                    name='status',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageattachment',
                    name='exporting_data_sub_stage',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageentity',
                    name='entity',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageentity',
                    name='exporting_data_sub_stage',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageuploaderclientlog',
                    name='attachment',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageuploaderclientlog',
                    name='entry',
                ),
                migrations.RemoveField(
                    model_name='exportingdatasubstageuploaderclientlog',
                    name='sub_stage',
                ),
                migrations.DeleteModel(
                    name='RegionalDataMartModelEnum',
                ),
                migrations.RemoveField(
                    model_name='transferredentity',
                    name='entity',
                ),
                migrations.RemoveField(
                    model_name='uploadstatusrequestlog',
                    name='entry',
                ),
                migrations.RemoveField(
                    model_name='uploadstatusrequestlog',
                    name='request_status',
                ),
                migrations.RemoveField(
                    model_name='uploadstatusrequestlog',
                    name='upload',
                ),
                migrations.DeleteModel(
                    name='UploaderClientLog',
                ),
                migrations.DeleteModel(
                    name='CollectingDataStageStatus',
                ),
                migrations.DeleteModel(
                    name='CollectingDataSubStageStatus',
                ),
                migrations.DeleteModel(
                    name='CollectingExportedDataStage',
                ),
                migrations.DeleteModel(
                    name='CollectingExportedDataSubStage',
                ),
                migrations.DeleteModel(
                    name='DataMartRequestStatus',
                ),
                migrations.DeleteModel(
                    name='ExportingDataStage',
                ),
                migrations.DeleteModel(
                    name='ExportingDataStageStatus',
                ),
                migrations.DeleteModel(
                    name='ExportingDataSubStage',
                ),
                migrations.DeleteModel(
                    name='ExportingDataSubStageAttachment',
                ),
                migrations.DeleteModel(
                    name='ExportingDataSubStageEntity',
                ),
                migrations.DeleteModel(
                    name='ExportingDataSubStageStatus',
                ),
                migrations.DeleteModel(
                    name='ExportingDataSubStageUploaderClientLog',
                ),
                migrations.DeleteModel(
                    name='RegionalDataMartEntityEnum',
                ),
                migrations.DeleteModel(
                    name='TransferredEntity',
                ),
                migrations.DeleteModel(
                    name='UploadStatusRequestLog',
                ),
            ],
        ),
    ]
