# Generated by Django 3.2.24 on 2025-04-17 12:43

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration', '0015_set_exporting_sub_stage_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='transferredentity',
            name='queue_level',
            field=models.PositiveIntegerField(
                choices=[(1, 'Быстрая'), (2, 'Основная'), (3, 'Долгая')], default=2, verbose_name='Уровень очереди'
            ),
        ),
    ]
